@@ -14,10 +14,11 @@ use Exporter;
 use SVG::XML;
 use SVG::Element;
 use SVG::Extension;
+use warnings;
 
 @ISA = qw(SVG::Element SVG::Extension);
 
-$VERSION = "2.34";
+$VERSION = "2.39";
 
 #-------------------------------------------------------------------------------
 
@@ -25,7 +26,7 @@ $VERSION = "2.34";
 
 =head2 VERSION
 
-Version 2.34, 15 August, 2007
+Version 2.39, 08  April, 2008
 
 Refer to L<SVG::Manual> for the complete manual
 
@@ -210,8 +211,8 @@ may also be set in xmlify, overriding any corresponding values set in the SVG->n
 # refer to the SVG::tag method 
 
 sub new ($;@) {
-    my ($proto,%attrs)=@_;
-    my $class=ref $proto || $proto;
+    my ($proto,%attrs) = @_;
+    my $class = ref $proto || $proto;
     my $self;
 
     # establish defaults for unspecified attributes
@@ -223,7 +224,10 @@ sub new ($;@) {
     unless ($attrs{-namespace}) {
         $attrs{'xmlns'} = $attrs{'xmlns'} || $attrs{'-xml_svg'};
     }
-    $attrs{'xmlns:xlink'} = $attrs{'xmlns:xlink'} || $attrs{'-xml_xlink'};
+    $attrs{'xmlns:xlink'} = $attrs{'xmlns:xlink'} || $attrs{'-xml_xlink'} || 'http://www.w3.org/1999/xlink';
+    $attrs{'xmlns:svg'} = $attrs{'xmlns:svg'} || $attrs{'-xml_svg'} || 'http://www.w3.org/2000/svg';
+
+
     $self->{-level} = 0;
     $self->{$_} = $attrs{$_} foreach keys %default_attrs;
 
