@@ -1,9 +1,8 @@
 package SVG::DOM;
 use strict;
-
+use warnings;
 use vars qw($VERSION);
-$VERSION = "1.01";
-#29.01.03 RO added setAttributes and setAttribute
+$VERSION = "1.04";
 
 # this module extends SVG::Element
 package SVG::Element;
@@ -384,6 +383,179 @@ sub getCDATA ($) {
 *getCdata=\&getCDATA;
 *getData=\&getCDATA;
 
+# ----------------
+# 2005-12-30 - Martin Owens, apply greater DOM specification (write)
+# http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html
+
+# ----------------
+# sub document
+sub document
+{
+	my ($self) = @_;
+	return $self->{-docref};
+}
+
+# DOM specified method names
+*createElement=\&tag;
+*firstChild=\&getFirstChild;
+*lastChild=\&getLastChild;
+*previousSibling=\&getPreviousSibling;
+*nextSibling=\&getNextSibling;
+
+# ----------------
+# sub insertBefore
+sub insertBefore
+{
+	my ($self, $newChild, $refChild) = @_;
+	return $self->appendElement($newChild) if not $refChild;
+	my $index = $self->findChildIndex($refChild);
+	return 0 if $index < 0; # NO_FOUND_ERR
+	return $self->insertAtIndex($newChild, $index);
+}
+*insertChildBefore=\&insertBefore;
+*insertNodeBefore=\&insertBefore;
+*insertElementBefore=\&insertBefore;
+
+# ----------------
+# sub insertAfter
+sub insertAfter
+{
+	my ($self, $newChild, $refChild) = @_;
+	return $self->appendElement($newChild) if not $refChild;
+	my $index = $self->findChildIndex($refChild);
+	return 0 if $index < 0; # NO_FOUND_ERR
+	return $self->insertAtIndex($newChild, $index+1);
+}
+*insertChildAfter=\&insertAfter;
+*insertNodeAfter=\&insertAfter;
+*insertElementAfter=\&insertAfter;
+
+# ----------------
+# sub insertSiblingAfter (Not in W3C DOM)
+sub insertSiblingAfter
+{
+	my ($self, $newChild) = @_;
+	return $self->parent->insertAfter($newChild, $self) if $self->parent;
+	return 0;
+}
+
+# ----------------
+# sub insertSiblingBefore (Not in W3C DOM)
+sub insertSiblingBefore
+{
+    my ($self, $newChild) = @_;
+    return $self->parent->insertBefore($newChild, $self) if $self->parent;
+    return 0;
+} 
+
+# ----------------
+# sub replaceChild
+sub replaceChild
+{
+	my ($self, $newChild, $oldChild) = @_;
+	# Replace newChild if it is in this list of children already
+	$self->removeChild($newChild) if $newChild->{-parent} eq $self;
+	# We need the index of the node to replace
+	my $index = $self->findChildIndex($oldChild);
+	return 0 if($index < 0); # NOT_FOUND_ERR
+	# Replace and bind new node with it's family
+	$self->removeChildAtIndex($index);
+	$self->insertChildAtIndex($index);
+	return $oldChild;
+}
+*replaceElement=\&replaceChild;
+*replaceNode=\&replaceChild;
+
+# ----------------
+# sub removeChild
+sub removeChild
+{
+	my ($self, $oldChild) = @_;
+	my $index = $self->findChildIndex($oldChild);
+	return 0 if($index < 0); # NOT_FOUND_ERR
+	return $self->removeChildAtIndex($index);
+}
+*removeElement=\&removeChild;
+*removeNode=\&removeChild;
+
+# ----------------
+# sub appendChild
+sub appendChild
+{
+    my ($self, $element) = @_;
+    $element->{-docref} = $self->{-docref};
+    $element->{-parent} = $self;
+    push @{$self->{-childs}}, $element;
+    return 1;
+}
+*appendElement=\&appendChild;
+*appendNode=\&appendChild;
+
+# ----------------
+# sub cloneNode
+sub cloneNode
+{
+	my ($self, $deep) = @_;
+	my $clone = new SVG::Element;
+	foreach my $key (keys(%{$self})) {
+		if($key ne '-childs' and $key ne '-parent') {
+			$clone->{$key} = $self->{$key};
+		}
+	}
+	# We need to clone the children if deep is specified.
+	if($deep) {
+		foreach my $child (@{$self->{-childs}}) {
+			my $childClone = $child->cloneNode($deep);
+			$clone->appendChild($childClone);
+		}
+	}
+	return $clone;
+}
+*cloneElement=\&cloneNode;
+
+# ---------------------------------------
+#    NONE DOM Supporting methodss
+
+# ----------------
+# sub findChildIndex
+sub findChildIndex
+{
+	my ($self, $refChild) = @_;
+	my $index;
+	foreach my $child (@{$self->{-childs}}) {
+        last if $child eq $refChild;
+        $index++;
+        return -1 if($index > @{$self->{-childs}}); 
+    } 
+	return $index;
+}
+
+# ---------------
+# sub insertAtIndex
+sub insertAtIndex
+{
+	my ($self, $newChild, $index) = @_;
+	splice @{$self->{-childs}}, $index, 0, $newChild;
+    $newChild->{-docref} = $self->{-docref};
+    $newChild->{-parent} = $self;
+	return 1;
+}
+
+# ----------------
+# sub removeChildAtIndex
+sub removeChildAtIndex
+{
+	my ($self, $index) = @_;
+	my $oldChild = splice @{$self->{-childs}}, $index, 1;
+	$oldChild->{-parent} = undef;
+    $oldChild->{-docref} = undef;
+	if(not @{$self->{-childs}}) {
+		delete $self->{-childs};
+	}
+	return $oldChild;
+} 
+
+
 #-------------------------------------------------------------------------------
 
 =pod 
@@ -398,9 +570,6 @@ SVG::DOM provides a selection of methods for accessing and manipulating SVG
 elements through DOM-like methods such as getElements, getChildren, getNextSibling
 and so on. 
 
-Currently only methods that provide read operations are supported. Methods to
-manipulate SVG elements will be added in a future release.
-
 =head1 SYNOPSIS
 
     my $svg=new SVG(id=>"svg_dom_synopsis", width=>"100", height=>"100");
@@ -429,6 +598,12 @@ manipulate SVG elements will be added in a future release.
     my @rectangles=$svg->getElements("rect");
     my $allelements_arrayref=$svg->getElements();
 
+	$group->insertBefore($newChild,$rect);
+	$group->insertAfter($newChild,$rect);
+	$rect = $group->replaceChild($newChild,$rect);
+	$group->removeChild($newChild);
+	my $newRect = $rect->cloneNode($deep);
+
     ...and so on...
 
 =head1 METHODS
@@ -539,16 +714,43 @@ Returns 1 if the current node is an ancestor of the specified element, otherwise
 
 Returns 1 if the current node is a descendant of the specified element, otherwise 0.
 
+=head2 $boolean = $obj->insertBefore( $element, $child );
+
+Returns 1 if $element was successfully inserted before $child in $obj
+
+=head2 $boolean = $obj->insertAfter( $element, $child );
+
+Returns 1 if $element was successfully inserted after $child in $obj
+
+=head2 $boolean = $obj->insertSiblingBefore( $element );
+
+Returns 1 if $element was successfully inserted before $obj
+
+=head2 $boolean = $obj->insertSiblingAfter( $element );
+
+Returns 1 if $element was successfully inserted after $obj 
+
+=head2 $element = $obj->replaceChild( $element, $child );
+
+Returns $child if $element successfully replaced $child in $obj
+
+=head2 $element = $obj->removeChild( $child );
+
+Returns $child if it was removed successfully from $obj
+
+=head2 $element = $obj->cloneNode( $deep ); 
+
+Returns a new $element clone of $obj, includes children if deep = 1
+
 =head1 AUTHOR
 
-Ronan Oger, ronan@roasp.com
+Ronan Oger, ronan@roitsystems.com
+Martin Owens, doctormo@postmaster.co.uk
 
 =head1 SEE ALSO
 
 perl(1), L<SVG>, L<SVG::XML>, L<SVG::Element>, L<SVG::Parser>, L<SVG::Manual>
 
-L<http://www.roasp.com/> ROASP.com: Serverside SVG server
-L<http://www.vectoreal.com/> Vectoreal: Commercal SVG Application solutions
 L<http://www.roitsystems.com/> ROIT Systems: Commercial SVG perl solutions
 L<http://www.w3c.org/Graphics/SVG/> SVG at the W3C
 
