@@ -1,3 +1,4 @@
+
 =pod 
 
 =head1 NAME
@@ -6,13 +7,12 @@ SVG::Element - Generate the element bits for SVG.pm
 
 =head1 AUTHOR
 
-Ronan Oger, ronan@roasp.com
+Ronan Oger, cpan@roitsystems.com
 
 =head1 SEE ALSO
 
 perl(1),L<SVG>,L<SVG::XML>,L<SVG::Element>,L<SVG::Parser>, L<SVG::Manual>
-L<http://www.roasp.com/> ROASP.com: Serverside SVG server
-L<http://www.vectoreal.com/> Vectoreal: Commercal SVG Application solutions
+L<http://www.roitsystems.com/> ROASP.com: Serverside SVG server
 L<http://www.roitsystems.com/> ROIT Systems: Commercial SVG perl solutions
 L<http://www.w3c.org/Graphics/SVG/> SVG at the W3C
 
@@ -20,65 +20,67 @@ L<http://www.w3c.org/Graphics/SVG/> SVG at the W3C
 
 package SVG::Element;
 
-$VERSION = "2.27";
+$VERSION = "2.38";
 
 use strict;
 use SVG::XML;
 use SVG::DOM;
 use SVG::Extension;
+use warnings;
 use vars qw($AUTOLOAD %autosubs);
 
-my @autosubs=qw(
-    animateMotion animateColor animateTransform circle ellipse rect polyline 
-    path polygon line title desc defs
-    altGlyph altGlyphDef altGlyphItem clipPath color-profile
-    cursor definition-src font-face-format font-face-name
-    font-face-src font-face-url foreignObject glyph
-    glyphRef hkern marker mask metadata missing-glyph
-    mpath switch symbol textPath tref tspan view vkern marker textbox
-    flowText style script
-    image a g
+my @autosubs = qw(
+  animateMotion animateColor animateTransform circle ellipse rect polyline
+  path polygon line title desc defs
+  altGlyph altGlyphDef altGlyphItem clipPath color-profile
+  cursor definition-src font-face-format font-face-name
+  font-face-src font-face-url foreignObject glyph
+  glyphRef hkern marker mask metadata missing-glyph
+  mpath switch symbol textPath tref tspan view vkern marker textbox
+  flowText style script
+  image a g
 );
 
-%autosubs=map { $_ => 1 } @autosubs;
+%autosubs = map { $_ => 1 } @autosubs;
 
 #-------------------------------------------------------------------------------
 
 sub new ($$;@) {
-    my ($proto,$name,%attrs)=@_;
-    my $class=ref($proto) || $proto;
-    my $self={-name => $name};
-    foreach my $key (keys %attrs) {
-	#handle escapes for special elements such as anchor
-        if ($key=~/^\-/) {
-            if ($key eq '-href') {
-	        $self->{'xlink:href'} = $attrs{$key};
-		$self->{'xlink:type'} = $attrs{-type} if $attrs{-type}; 
-		$self->{'xlink:role'} = $attrs{-role} if $attrs{-role};
-	        $self->{'xlink:title'} = $attrs{-title} if $attrs{-title};
-		$self->{'xlink:show'} = $attrs{-show} if $attrs{-show};
-		$self->{'xlink:arcrole'} = $attrs{-arcrole} if $attrs{-arcrole};
-		$self->{'xlink:actuate'} = $attrs{-actuate} if $attrs{-actuate};
-        	next;
+    my ( $proto, $name, %attrs ) = @_;
+    my $class = ref($proto) || $proto;
+    my $self = { -name => $name };
+    foreach my $key ( keys %attrs ) {
+
+        #handle escapes for special elements such as anchor
+        if ( $key =~ /^\-/ ) {
+            if ( $key eq '-href' ) {
+                $self->{'xlink:href'}    = $attrs{$key};
+                $self->{'xlink:type'}    = $attrs{-type} if $attrs{-type};
+                $self->{'xlink:role'}    = $attrs{-role} if $attrs{-role};
+                $self->{'xlink:title'}   = $attrs{-title} if $attrs{-title};
+                $self->{'xlink:show'}    = $attrs{-show} if $attrs{-show};
+                $self->{'xlink:arcrole'} = $attrs{-arcrole} if $attrs{-arcrole};
+                $self->{'xlink:actuate'} = $attrs{-actuate} if $attrs{-actuate};
+                next;
             }
-	} 
-        $self->{$key}=$attrs{$key};
+        }
+        $self->{$key} = $attrs{$key};
     }
 
-    return bless($self,$class);
+    return bless( $self, $class );
 }
 
 #-------------------------------------------------------------------------------
 
 sub release ($) {
-    my $self=shift;
+    my $self = shift;
 
-    foreach my $key (keys(%{$self})) {
-        next if $key=~/^\-/;
-        if (ref($self->{$key})=~/^SVG/) {
+    foreach my $key ( keys( %{$self} ) ) {
+        next if $key =~ /^\-/;
+        if ( ref( $self->{$key} ) =~ /^SVG/ ) {
             eval { $self->{$key}->release; };
         }
-        delete($self->{$key});
+        delete( $self->{$key} );
     }
 
     return $self;
@@ -86,127 +88,139 @@ sub release ($) {
 
 sub xmlify ($) {
     my $self = shift;
-    my $ns = $self->{-namespace} || $self->{-docref}->{-namespace} || undef;
-    my $xml = '';
+    my $ns   = $self->{-namespace} || $self->{-docref}->{-namespace} || undef;
+    my $xml  = '';
+
     #prep the attributes
     my %attrs;
-    foreach my $k (keys(%{$self})) {
-        if($k=~/^\-/) { next; }
-        if(ref($self->{$k}) eq 'ARRAY') {
-            $attrs{$k}=join(', ',@{$self->{$k}});
-        } elsif(ref($self->{$k}) eq 'HASH') {
-            $attrs{$k}=cssstyle(%{$self->{$k}});
-        } elsif(ref($self->{$k}) eq '') {
-            $attrs{$k}=$self->{$k};
+    foreach my $k ( keys( %{$self} ) ) {
+        if ( $k =~ /^\-/ ) { next; }
+        if ( ref( $self->{$k} ) eq 'ARRAY' ) {
+            $attrs{$k} = join( ', ', @{ $self->{$k} } );
+        } elsif ( ref( $self->{$k} ) eq 'HASH' ) {
+            $attrs{$k} = cssstyle( %{ $self->{$k} } );
+        } elsif ( ref( $self->{$k} ) eq '' ) {
+            $attrs{$k} = $self->{$k};
         }
     }
+
     #prep the tag
-    if($self->{-comment}) {
-        $xml .= $self->xmlcomment($self->{-comment});
+    if ( $self->{-comment} ) {
+        $xml .= $self->xmlcomment( $self->{-comment} );
         return $xml;
-    } elsif ($self->{-name} eq 'document') {
+    } elsif ( $self->{-name} eq 'document' ) {
+
         #write the xml header
         $xml .= $self->xmldecl;
-        $xml .= $self->xmlpi($self->{-document}->{-pi}) if $self->{-document}->{-pi};
+        $xml .= $self->xmlpi( $self->{-document}->{-pi} )
+          if $self->{-document}->{-pi};
+
         #and write the dtd if this is inline
         $xml .= $self->dtddecl unless $self->{-inline};
-        foreach my $k (@{$self->{-childs}}) {
-            if(ref($k)=~/^SVG::Element/) {
+        foreach my $k ( @{ $self->{-childs} } ) {
+            if ( ref($k) =~ /^SVG::Element/ ) {
                 $xml .= $k->xmlify($ns);
             }
         }
         return $xml;
-    } 
-    if(defined $self->{-childs} ||
-        defined $self->{-cdata} ||
-        defined $self->{-CDATA} ||
-        defined $self->{-cdata_noxmlesc}) {
-        $xml .= $self->{-docref}->{-elsep};
+    }
+    if (
+        defined $self->{-childs}
+        || defined $self->{-cdata}
+        ||defined $self->{-CDATA}
+        ||defined $self->{-cdata_noxmlesc}
+      )
+    {
+        $xml .= $self->{-docref}->{-elsep} if defined $self->{-childs};
         $xml .= $self->{-docref}->{-indent} x $self->{-docref}->{-level};
-        $xml .= xmltagopen_ln($self->{-name},$ns,%attrs);
+        $xml .= xmltagopen_ln( $self->{-name}, $ns, %attrs );
         $self->{-docref}->{-level}++;
-        foreach my $k (@{$self->{-childs}}) {
-            if(ref($k)=~/^SVG::Element/) {
+        foreach my $k ( @{ $self->{-childs} } ) {
+            if ( ref($k) =~ /^SVG::Element/ ) {
                 $xml .= $k->xmlify($ns);
             }
         }
 
-        if(defined $self->{-cdata}) {
-            $xml .= xmlescp($self->{-cdata});
-        } 
-        if(defined $self->{-CDATA}) {
-            $xml .= '<![CDATA['.$self->{-CDATA}.']]>';
+        if ( defined $self->{-cdata} ) {
+            $xml .= xmlescp( $self->{-cdata} );
         }
-        if(defined $self->{-cdata_noxmlesc}) {
+        if ( defined $self->{-CDATA} ) {
+            $xml .= '<![CDATA[' . $self->{-CDATA} . ']]>';
+        }
+        if ( defined $self->{-cdata_noxmlesc} ) {
             $xml .= $self->{-cdata_noxmlesc};
         }
 
-
         #return without writing the tag out if it the document tag
         $self->{-docref}->{-level}--;
         $xml .= $self->{-docref}->{-elsep};
         $xml .= $self->{-docref}->{-indent} x $self->{-docref}->{-level};
-        $xml .= xmltagclose_ln($self->{-name},$ns);
+        $xml .= xmltagclose_ln( $self->{-name}, $ns );
     } else {
         $xml .= $self->{-docref}->{-elsep};
         $xml .= $self->{-docref}->{-indent} x $self->{-docref}->{-level};
-        $xml .= xmltag_ln($self->{-name},$ns,%attrs);
+        $xml .= xmltag_ln( $self->{-name}, $ns, %attrs );
     }
+
     #return the finished tag
     return $xml;
 }
 
 sub perlify {
-    my $self=shift;
-    my $code='';
+    my $self = shift;
+    my $code = '';
 
     #prep the attributes
     my %attrs;
-    foreach my $k (keys(%{$self})) {
-        if($k=~/^\-/) { next; }
-        if(ref($self->{$k}) eq 'ARRAY') {
-            $attrs{$k}=join(', ',@{$self->{$k}});
-        } elsif(ref($self->{$k}) eq 'HASH') {
-            $attrs{$k}=cssstyle(%{$self->{$k}});
-        } elsif(ref($self->{$k}) eq '') {
-            $attrs{$k}=$self->{$k};
+    foreach my $k ( keys( %{$self} ) ) {
+        if ( $k =~ /^\-/ ) { next; }
+        if ( ref( $self->{$k} ) eq 'ARRAY' ) {
+            $attrs{$k} = join( ', ', @{ $self->{$k} } );
+        } elsif ( ref( $self->{$k} ) eq 'HASH' ) {
+            $attrs{$k} = cssstyle( %{ $self->{$k} } );
+        } elsif ( ref( $self->{$k} ) eq '' ) {
+            $attrs{$k} = $self->{$k};
         }
     }
 
-    if($self->{-comment}) {
+    if ( $self->{-comment} ) {
         $code .= "->comment($self->{-comment})";
         return $code;
-    } elsif($self->{-pi}) {
+    } elsif ( $self->{-pi} ) {
         $code .= "->pi($self->{-pi})";
         return $code;
-    } elsif ($self->{-name} eq 'document') {
+    } elsif ( $self->{-name} eq 'document' ) {
+
         #write the xml header
         #$xml .= $self->xmldecl;
         #and write the dtd if this is inline
         #$xml .= $self->dtddecl unless $self->{-inline};
-        foreach my $k (@{$self->{-childs}}) {
-            if(ref($k)=~/^SVG::Element/) {
+        foreach my $k ( @{ $self->{-childs} } ) {
+            if ( ref($k) =~ /^SVG::Element/ ) {
                 $code .= $k->perlify();
             }
         }
         return $code;
     }
 
-    if (defined $self->{-childs}) {
+    if ( defined $self->{-childs} ) {
         $code .= $self->{-docref}->{-elsep};
         $code .= $self->{-docref}->{-indent} x $self->{-docref}->{-level};
-        $code .= $self->{-name}.'('.(join ', ',(map { "$_=>'$attrs{$_}'"} sort keys %attrs)).')';
-        if ($self->{-cdata}) {
-            $code.="->cdata($self->{-cdata})";
-        } elsif ($self->{-CDATA}) {
-            $code.="->CDATA($self->{-CDATA})";
-        } elsif ($self->{-cdata_noxmlesc}) {
-            $code.="->cdata_noxmlesc($self->{-cdata_noxmlesc})";
+        $code .=
+            $self->{-name} . '('
+          . ( join ', ', ( map { "$_=>'$attrs{$_}'" } sort keys %attrs ) )
+          . ')';
+        if ( $self->{-cdata} ) {
+            $code .= "->cdata($self->{-cdata})";
+        } elsif ( $self->{-CDATA} ) {
+            $code .= "->CDATA($self->{-CDATA})";
+        } elsif ( $self->{-cdata_noxmlesc} ) {
+            $code .= "->cdata_noxmlesc($self->{-cdata_noxmlesc})";
         }
 
         $self->{-docref}->{-level}++;
-        foreach my $k (@{$self->{-childs}}) {
-            if(ref($k)=~/^SVG::Element/) {
+        foreach my $k ( @{ $self->{-childs} } ) {
+            if ( ref($k) =~ /^SVG::Element/ ) {
                 $code .= $k->perlify();
             }
         }
@@ -214,17 +228,19 @@ sub perlify {
     } else {
         $code .= $self->{-docref}->{-elsep};
         $code .= $self->{-docref}->{-indent} x $self->{-docref}->{-level};
-        $code .= $self->{-name}.'('.(join ', ',(map { "$_=>'$attrs{$_}'"} sort keys %attrs)).')';
+        $code .=
+            $self->{-name} . '('
+          . ( join ', ', ( map { "$_=>'$attrs{$_}'" } sort keys %attrs ) )
+          . ')';
     }
 
     return $code;
 }
-*toperl=\&perlify;
-
+*toperl = \&perlify;
 
 sub addchilds ($@) {
-    my $self=shift;
-    push @{$self->{-childs}},@_;
+    my $self = shift;
+    push @{ $self->{-childs} }, @_;
     return $self;
 }
 
@@ -245,60 +261,61 @@ B<Example:>
 =cut
 
 sub tag ($$;@) {
-    my ($self,$name,%attrs)=@_;
+    my ( $self, $name, %attrs ) = @_;
 
-    unless ($self->{-parent}) {
-      #traverse down the tree until you find a non-document entry
-      while ($self->{-document})  {$self = $self->{-document}}
+    unless ( $self->{-parent} ) {
+
+        #traverse down the tree until you find a non-document entry
+        while ( $self->{-document} ) { $self = $self->{-document} }
     }
-    my $tag=new SVG::Element($name,%attrs);
+    my $tag = new SVG::Element( $name, %attrs );
 
     #define the element namespace
-    $tag->{-namespace}=$attrs{-namespace} if ($attrs{-namespace});
+    $tag->{-namespace} = $attrs{-namespace} if ( $attrs{-namespace} );
 
     #add the tag to the document element
     $tag->{-docref} = $self->{-docref};
-    
+
     #create the empty idlist hash ref unless it already exists
-    $tag->{-docref}->{-idlist} = {} 
-        unless (defined $tag->{-docref}->{-idlist});
-    
+    $tag->{-docref}->{-idlist} = {}
+      unless ( defined $tag->{-docref}->{-idlist} );
+
     #verify that the current id is unique. compain on exception
     #>>>TBD: add -strictids option to disable this check if desired
-    if ($tag->{id}) {
-        if ($self->getElementByID($tag->{id})) {
-            $self->error($tag->{id} => "ID already exists in document");
+    if ( $tag->{id} ) {
+        if ( $self->getElementByID( $tag->{id} ) ) {
+            $self->error( $tag->{id} => "ID already exists in document" );
             return undef;
         }
     }
 
     #add the current id reference to the document id hash
-    $tag->{-docref}->{-idlist}->{$tag->{id}} = $tag if defined ($tag->{id});
-    
-    #create the empty idlist hash ref unless it already exists
-    $tag->{-docref}->{-elist} = {} 
-        unless (defined $tag->{-docref}->{-elist});
-    
-    #create the empty idlist hash ref unless it already exists
-    $tag->{-docref}->{-elist}->{$tag->{-name}} = [] 
-        unless (defined $tag->{-docref}->{-elist}->{$tag->{-name}});
+    $tag->{-docref}->{-idlist}->{ $tag->{id} } = $tag if defined( $tag->{id} );
 
-    #add the current element ref to the corresponding element-hash array
-    # -elist is a hash of element names. key name is element, content is object ref.
+    #create the empty idlist hash ref unless it already exists
+    $tag->{-docref}->{-elist} = {}
+      unless ( defined $tag->{-docref}->{-elist} );
+
+    #create the empty idlist hash ref unless it already exists
+    $tag->{-docref}->{-elist}->{ $tag->{-name} } = []
+      unless ( defined $tag->{-docref}->{-elist}->{ $tag->{-name} } );
+
+#add the current element ref to the corresponding element-hash array
+# -elist is a hash of element names. key name is element, content is object ref.
 
     # add the reference to $tag to the array of refs that belong to the
     # key $tag->{-name}.
-    unshift    @{$tag->{-docref}->{-elist}->{$tag->{-name}}},$tag;
+    unshift @{ $tag->{-docref}->{-elist}->{ $tag->{-name} } }, $tag;
 
-        # attach element to the DOM of the document
-    $tag->{-parent}=$self;
-    $tag->{-parentname}=$self->{-name};
+    # attach element to the DOM of the document
+    $tag->{-parent}     = $self;
+    $tag->{-parentname} = $self->{-name};
     $self->addchilds($tag);
 
-    return($tag);
+    return ($tag);
 }
 
-*element=\&tag;
+*element = \&tag;
 
 =pod
 
@@ -352,19 +369,20 @@ for more information about the options above, refer to Link  section in the SVG
 =cut
 
 sub anchor {
-    my ($self,%attrs)=@_;
-    my $an=$self->tag('a',%attrs);
+    my ( $self, %attrs ) = @_;
+    my $an = $self->tag( 'a', %attrs );
+
     #$an->{'xlink:href'}=$attrs{-href} if(defined $attrs{-href});
     #$an->{'target'}=$attrs{-target} if(defined $attrs{-target});
-    return($an);
+    return ($an);
 }
 
 sub svg {
-    my ($self,%attrs)=@_;
-    my $svg=$self->tag('svg',%attrs);
-    $svg->{'height'} = '100%' unless ($svg->{'height'});
-    $svg->{'width'}  = '100%' unless ($svg->{'width'});
-    return($svg);
+    my ( $self, %attrs ) = @_;
+    my $svg = $self->tag( 'svg', %attrs );
+    $svg->{'height'} = '100%' unless ( $svg->{'height'} );
+    $svg->{'width'}  = '100%' unless ( $svg->{'width'} );
+    return ($svg);
 }
 
 =pod
@@ -427,8 +445,8 @@ B<Example:>
 =cut
 
 sub rectangle ($;@) {
-    my ($self,%attrs)=@_;
-        return $self->tag('rect',%attrs);
+    my ( $self, %attrs ) = @_;
+    return $self->tag( 'rect', %attrs );
 }
 
 =pod
@@ -490,9 +508,9 @@ single element within an external SVG file.
 =cut
 
 sub use ($;@) {
-    my ($self,%attrs)=@_;
-    my $u=$self->tag('use',%attrs);
-    $u->{'xlink:href'}=$attrs{-href} if(defined $attrs{-href});
+    my ( $self, %attrs ) = @_;
+    my $u = $self->tag( 'use', %attrs );
+    $u->{'xlink:href'} = $attrs{-href} if ( defined $attrs{-href} );
     return $u;
 }
 
@@ -613,18 +631,20 @@ SEE ALSO:
 =cut
 
 sub text ($;@) {
-    my ($self,%attrs)=@_;
+    my ( $self, %attrs ) = @_;
     my $pre = '';
     $pre = $attrs{-type} || 'std';
-    my %get_pre = (std=>'text',
-                   path=>'textPath',
-                   span=>'tspan',);
+    my %get_pre = (
+        std  => 'text',
+        path => 'textPath',
+        span => 'tspan',
+    );
 
-    $pre = $get_pre{lc($pre)};
-    my $text=$self->tag($pre,%attrs);
-    $text->{'xlink:href'} = $attrs{-href} if(defined $attrs{-href});
-       $text->{'target'} = $attrs{-target} if(defined $attrs{-target});
-    return($text);
+    $pre = $get_pre{ lc($pre) };
+    my $text = $self->tag( $pre, %attrs );
+    $text->{'xlink:href'} = $attrs{-href}   if ( defined $attrs{-href} );
+    $text->{'target'}     = $attrs{-target} if ( defined $attrs{-target} );
+    return ($text);
 }
 
 =pod
@@ -666,9 +686,9 @@ B<Example:>
 =cut
 
 sub comment ($;@) {
-    my ($self,@text)=@_;
+    my ( $self, @text ) = @_;
     my $tag = $self->tag('comment');
-    $tag->{-comment} = [@text];
+    $tag->{ -comment } = [@text];
     return $tag;
 }
 
@@ -690,17 +710,17 @@ B<Example:>
 
 =cut
 
-#add 
+#add
 
 sub pi ($;@) {
-    my ($self,@text)=@_;
-    return $self->{-document}->{-pi} unless scalar @text;
-    my @pi; 
-    @pi = @{$self->{-document}->{-pi}} if $self->{-document}->{-pi};	
-    unshift(@text,@pi) if @pi;
-    $self->{-document}->{-pi} = \@text;
+    my ( $self, @text ) = @_;
+    return $self->{-document}->{ -pi } unless scalar @text;
+    my @pi;
+    @pi = @{ $self->{-document}->{ -pi } } if $self->{-document}->{ -pi };
+    unshift( @text, @pi ) if @pi;
+    $self->{-document}->{ -pi } = \@text;
     my $tag = $self->tag('pi');
-    return $tag
+    return $tag;
 }
 
 =pod
@@ -842,49 +862,57 @@ B<Aliases:> get_path set_path
 =cut
 
 sub get_path ($;@) {
-    my ($self,%attrs) = @_;
+    my ( $self, %attrs ) = @_;
 
     my $type = $attrs{-type} || 'path';
-    my @x = @{$attrs{x}};
-    my @y = @{$attrs{y}};
+    my @x    = @{ $attrs{x} };
+    my @y    = @{ $attrs{y} };
     my $points;
+
     # we need a path-like point string returned
-    if (lc($type) eq 'path') {
+    if ( lc($type) eq 'path' ) {
         my $char = 'M';
-        $char = ' m ' if (defined $attrs{-relative} && lc($attrs{-relative}));
+        $char = ' m '
+          if ( defined $attrs{-relative} && lc( $attrs{-relative} ) );
         while (@x) {
+
             #scale each value
             my $x = shift @x;
             my $y = shift @y;
+
             #append the scaled value to the graph
             $points .= "$char $x $y ";
             $char = ' L ';
-            $char = ' l ' if (defined $attrs{-relative}
-                                && lc($attrs{-relative}));
+            $char = ' l '
+              if ( defined $attrs{-relative}
+                && lc( $attrs{-relative} ) );
         }
-        $points .=  ' z ' if (defined $attrs{-closed} && lc($attrs{-closed}));
-        my %out = (d => $points);
+        $points .= ' z '
+          if ( defined $attrs{-closed} && lc( $attrs{-closed} ) );
+        my %out = ( d => $points );
         return \%out;
-    } elsif (lc($type) =~ /^poly/){
+    } elsif ( lc($type) =~ /^poly/ ) {
         while (@x) {
+
             #scale each value
             my $x = shift @x;
             my $y = shift @y;
+
             #append the scaled value to the graph
             $points .= "$x,$y ";
         }
     }
-    my %out = (points=>$points);
+    my %out = ( points => $points );
     return \%out;
 }
 
 sub make_path ($;@) {
-    my ($self,%attrs) = @_;
+    my ( $self, %attrs ) = @_;
     return get_path(%attrs);
 }
 
 sub set_path ($;@) {
-    my ($self,%attrs) = @_;
+    my ( $self, %attrs ) = @_;
     return get_path(%attrs);
 }
 
@@ -934,60 +962,55 @@ B<Inputs:> -method = Transform | Motion | Color
 =cut
 
 sub animate ($;@) {
-    my ($self,%attrs) = @_;
-    my %rtr = %attrs;
-    my $method = $rtr{'-method'}; # Set | Transform | Motion | Color
+    my ( $self, %attrs ) = @_;
+    my %rtr    = %attrs;
+    my $method = $rtr{'-method'};    # Set | Transform | Motion | Color
 
     $method = lc($method);
 
     # we do not want this to pollute the generation of the tag
-    delete $rtr{-method};  #bug report from briac.
+    delete $rtr{-method};            #bug report from briac.
 
     my %animation_method = (
-        transform=>'animateTransform',
-        motion=>'animateMotion',
-        color=>'animateColor',
-        set=>'set',
-        attribute=>'animate'
+        transform => 'animateTransform',
+        motion    => 'animateMotion',
+        color     => 'animateColor',
+        set       => 'set',
+        attribute => 'animate'
     );
-    
+
     my $name = $animation_method{$method} || 'animate';
-    
+
     #list of legal entities for each of the 5 methods of animations
     my %legal = (
-        animate =>    
-          qq begin dur  end  min  max  restart  repeatCount 
+        animate => qq begin dur  end  min  max  restart  repeatCount 
               repeatDur  fill  attributeType attributeName additive
               accumulate calcMode  values  keyTimes  keySplines
               from  to  by ,
-        animateTransform =>    
-          qq begin dur  end  min  max  restart  repeatCount
+        animateTransform => qq begin dur  end  min  max  restart  repeatCount
               repeatDur  fill  additive  accumulate calcMode  values
               keyTimes  keySplines  from  to  by calcMode path keyPoints
               rotate origin type attributeName attributeType ,
-    	animateMotion =>    
-          qq begin dur  end  min  max  restart  repeatCount
+        animateMotion => qq begin dur  end  min  max  restart  repeatCount
               repeatDur  fill  additive  accumulate calcMode  values
               to  by keyTimes keySplines  from  path  keyPoints
               rotate  origin ,
-        animateColor =>    
-          qq begin dur  end  min  max  restart  repeatCount
+        animateColor => qq begin dur  end  min  max  restart  repeatCount
               repeatDur  fill  additive  accumulate calcMode  values
               keyTimes  keySplines  from  to  by ,
-        set =>    
-          qq begin dur  end  min  max  restart  repeatCount  repeatDur
+        set => qq begin dur  end  min  max  restart  repeatCount  repeatDur
               fill to 
     );
 
-    foreach my $k (keys %rtr) {
-        next if ($k =~ /\-/);
+    foreach my $k ( keys %rtr ) {
+        next if ( $k =~ /\-/ );
 
-        if ($legal{$name} !~ /\b$k\b/) {
-            $self->error("$name.$k" => "Illegal animation command");
+        if ( $legal{$name} !~ /\b$k\b/ ) {
+            $self->error( "$name.$k" => "Illegal animation command" );
         }
     }
 
-    return $self->tag($name,%rtr);
+    return $self->tag( $name, %rtr );
 }
 
 =pod
@@ -1014,8 +1037,8 @@ B<Example:>
 =cut
 
 sub group ($;@) {
-    my ($self,%attrs)=@_;
-    return $self->tag('g',%attrs);
+    my ( $self, %attrs ) = @_;
+    return $self->tag( 'g', %attrs );
 }
 
 =pod
@@ -1041,11 +1064,11 @@ which the value of the property is not redefined by the child.
 =cut
 
 sub STYLE ($;@) {
-    my ($self,%attrs)=@_;
+    my ( $self, %attrs ) = @_;
 
-    $self->{style}=$self->{style} || {};
-    foreach my $k (keys %attrs) {
-        $self->{style}->{$k}=$attrs{$k};
+    $self->{style} = $self->{style} || {};
+    foreach my $k ( keys %attrs ) {
+        $self->{style}->{$k} = $attrs{$k};
     }
 
     return $self;
@@ -1062,11 +1085,11 @@ Sets/Adds mouse action definitions for tag
 =cut
 
 sub mouseaction ($;@) {
-    my ($self,%attrs)=@_;
+    my ( $self, %attrs ) = @_;
 
-    $self->{mouseaction}=$self->{mouseaction} || {};
-    foreach my $k (keys %attrs) {
-        $self->{mouseaction}->{$k}=$attrs{$k};
+    $self->{mouseaction} = $self->{mouseaction} || {};
+    foreach my $k ( keys %attrs ) {
+        $self->{mouseaction}->{$k} = $attrs{$k};
     }
 
     return $self;
@@ -1103,33 +1126,36 @@ B<Aliases:> attr attribute
 =cut
 
 sub attrib ($$;$) {
-    my ($self,$name,$val)=@_;
+    my ( $self, $name, $val ) = @_;
 
     #verify that the current id is unique. compain on exception
-    if ($name eq "id") {
-        if ($self->getElementByID($val)) {
-            $self->error($val => "ID already exists in document");
+    if ( $name eq "id" ) {
+        if ( $self->getElementByID($val) ) {
+            $self->error( $val => "ID already exists in document" );
             return undef;
         }
-    } 
+    }
+
+    if ( not defined $val ) {
+        if ( scalar(@_) == 2 ) {
 
-    if (not defined $val) {
-        if (scalar(@_)==2) {
             # two arguments only - retrieve
             return $self->{$name};
         } else {
+
             # 3rd argument is undef - delete
             delete $self->{$name};
         }
     } else {
+
         # 3 defined arguments - set
-        $self->{$name}=$val;
+        $self->{$name} = $val;
     }
 
     return $self;
 }
-*attr=\&attrib;
-*attribute=\&attrib;
+*attr      = \&attrib;
+*attribute = \&attrib;
 
 =pod
 
@@ -1165,9 +1191,9 @@ SEE ALSO:
 =cut
 
 sub cdata ($@) {
-    my ($self,@txt)=@_;
-    $self->{-cdata}=join(' ',@txt);
-    return($self);
+    my ( $self, @txt ) = @_;
+    $self->{ -cdata } = join( ' ', @txt );
+    return ($self);
 }
 
 =pod
@@ -1238,15 +1264,15 @@ SEE ALSO:
 =cut
 
 sub CDATA ($@) {
-    my ($self,@txt)=@_;
-    $self->{-CDATA}=join('\n',@txt);
-    return($self);
+    my ( $self, @txt ) = @_;
+    $self->{ -CDATA } = join( '\n', @txt );
+    return ($self);
 }
 
 sub cdata_noxmlesc ($@) {
-    my ($self,@txt)=@_;
-    $self->{-cdata_noxmlesc}=join('\n',@txt);
-    return($self);
+    my ( $self, @txt ) = @_;
+    $self->{ -cdata_noxmlesc } = join( '\n', @txt );
+    return ($self);
 }
 
 =pod
@@ -1277,8 +1303,8 @@ L<"fe">.
 =cut
 
 sub filter ($;@) {
-    my ($self,%attrs)=@_;
-    return $self->tag('filter',%attrs);
+    my ( $self, %attrs ) = @_;
+    return $self->tag( 'filter', %attrs );
 }
 
 =pod
@@ -1364,42 +1390,42 @@ L<"filter">.
 =cut
 
 sub fe ($;@) {
-    my ($self,%attrs) = @_;
+    my ( $self, %attrs ) = @_;
 
-    return 0 unless  ($attrs{'-type'});
+    return 0 unless ( $attrs{'-type'} );
     my %allowed = (
-        blend => 'feBlend',
-        colormatrix => 'feColorMatrix',
-        componenttrans => 'feComponentTrans',
-        Componenttrans => 'feComponentTrans',
-        composite => 'feComposite',
-        convolvematrix => 'feConvolveMatrix',
-        diffuselighting => 'feDiffuseLighting',
-        displacementmap => 'feDisplacementMap',
-        distantlight => 'feDistantLight',
-        flood => 'feFlood',
-        funca => 'feFuncA',
-        funcb => 'feFuncB',
-        funcg => 'feFuncG',
-        funcr => 'feFuncR',
-        gaussianblur => 'feGaussianBlur',
-        image => 'feImage',
-        merge => 'feMerge',
-        mergenode => 'feMergeNode',
-        morphology => 'feMorphology',
-        offset => 'feOffset',
-        pointlight => 'fePointLight',
+        blend            => 'feBlend',
+        colormatrix      => 'feColorMatrix',
+        componenttrans   => 'feComponentTrans',
+        Componenttrans   => 'feComponentTrans',
+        composite        => 'feComposite',
+        convolvematrix   => 'feConvolveMatrix',
+        diffuselighting  => 'feDiffuseLighting',
+        displacementmap  => 'feDisplacementMap',
+        distantlight     => 'feDistantLight',
+        flood            => 'feFlood',
+        funca            => 'feFuncA',
+        funcb            => 'feFuncB',
+        funcg            => 'feFuncG',
+        funcr            => 'feFuncR',
+        gaussianblur     => 'feGaussianBlur',
+        image            => 'feImage',
+        merge            => 'feMerge',
+        mergenode        => 'feMergeNode',
+        morphology       => 'feMorphology',
+        offset           => 'feOffset',
+        pointlight       => 'fePointLight',
         specularlighting => 'feSpecularLighting',
-        spotlight => 'feSpotLight',
-        tile => 'feTile',
-        turbulence => 'feTurbulence'
+        spotlight        => 'feSpotLight',
+        tile             => 'feTile',
+        turbulence       => 'feTurbulence'
     );
 
-    my $key = lc($attrs{'-type'});
-    my $fe_name = $allowed{lc($key)} || 'error:illegal_filter_element';
+    my $key = lc( $attrs{'-type'} );
+    my $fe_name = $allowed{ lc($key) } || 'error:illegal_filter_element';
     delete $attrs{'-type'};
 
-    return $self->tag($fe_name, %attrs);
+    return $self->tag( $fe_name, %attrs );
 }
 
 =pod
@@ -1423,8 +1449,8 @@ B<Example:>
 =cut
 
 sub pattern ($;@) {
-    my ($self,%attrs)=@_;
-    return $self->tag('pattern',%attrs);
+    my ( $self, %attrs ) = @_;
+    return $self->tag( 'pattern', %attrs );
 }
 
 =pod
@@ -1449,8 +1475,8 @@ B<Example:>
 =cut
 
 sub set ($;@) {
-    my ($self,%attrs)=@_;
-    return $self->tag('set',%attrs);
+    my ( $self, %attrs ) = @_;
+    return $self->tag( 'set', %attrs );
 }
 
 =pod
@@ -1474,8 +1500,8 @@ B<Example:>
 =cut
 
 sub stop ($;@) {
-    my ($self,%attrs)=@_;
-    return $self->tag('stop',%attrs);
+    my ( $self, %attrs ) = @_;
+    return $self->tag( 'stop', %attrs );
 }
 
 =pod
@@ -1494,15 +1520,15 @@ B<Example:>
 =cut
 
 sub gradient ($;@) {
-    my ($self,%attrs)=@_;
+    my ( $self, %attrs ) = @_;
 
     my $type = $attrs{'-type'} || 'linear';
-    unless ($type =~ /^(linear|radial)$/) {
+    unless ( $type =~ /^(linear|radial)$/ ) {
         $type = 'linear';
     }
     delete $attrs{'-type'};
 
-    return $self->tag($type.'Gradient',%attrs);
+    return $self->tag( $type . 'Gradient', %attrs );
 }
 
 =pod
@@ -1573,43 +1599,49 @@ See e.g. L<"pattern"> for an example of the use of these methods.
 # Internal methods
 
 sub error ($$$) {
-    my ($self,$command,$error)=@_;
+    my ( $self, $command, $error ) = @_;
 
-    if ($self->{-docref}->{-raiseerror}) {
+    if ( $self->{-docref}->{-raiseerror} ) {
         die "$command: $error\n";
-    } elsif ($self->{-docref}->{-printerror}) {
+    } elsif ( $self->{-docref}->{-printerror} ) {
         print STDERR "$command: $error\n";
     }
 
-    $self->{errors}{$command}=$error;
+    $self->{errors}{$command} = $error;
 }
 
 # This AUTOLOAD method is activated when '-auto' is passed to SVG.pm
 sub autoload {
-    my $self=shift;
-    my ($package,$sub)=($AUTOLOAD=~/(.*)::([^:]+)$/);
+    my $self = shift;
+    my ( $package, $sub ) = ( $AUTOLOAD =~ /(.*)::([^:]+)$/ );
 
-    if ($sub eq 'DESTROY') {
+    if ( $sub eq 'DESTROY' ) {
         return $self->release();
     } else {
+
         # the import routine may call us with a tag name involving '-'s
-        my $tag=$sub; $sub=~tr/-/_/;
+        my $tag = $sub;
+        $sub =~ tr/-/_/;
+
         # N.B.: The \ on \@_ makes sure that the incoming arguments are
         # used and not the ones passed when the subroutine was created.
-        eval "sub $package\:\:$sub (\$;\@) { return shift->tag('$tag',\@_) }";
+       # eval "sub $package\:\:$sub (\$;\@) { return shift->tag('$tag',\@_) }";
+	#per rt.perl.org comment by slaven.
+
+	if (!$package->can($sub)) {
+		no strict 'refs';
+		*{$package.'::'.$sub} = sub { return shift->tag($tag, @_) };
+	}
         return $self->$sub(@_) if $self;
     }
 }
 
-
-
-
 #-------------------------------------------------------------------------------
 # GD Routines
 
 sub colorAllocate ($$$$) {
-    my ($self,$red,$green,$blue)=@_;
-    return 'rgb('.int($red).','.int($green).','.int($blue).')';
+    my ( $self, $red, $green, $blue ) = @_;
+    return 'rgb(' . int($red) . ',' . int($green) . ',' . int($blue) . ')';
 }
 
 #-------------------------------------------------------------------------------
